@@ -13,6 +13,10 @@
 #include <QQueue>
 #include <qmath.h>
 #include <QSettings>
+#include <QAccelerometer>
+#include <paccelerometer.h>
+
+QTM_USE_NAMESPACE
 
 #include "OverlayWidget.h"
 #include "CameraParameters.h"
@@ -82,15 +86,34 @@ void CameraThread::run() {
     // histograms or sharpness), image memory auto allocated for each
     // new photograph, so that we can have multiple unique photographs
     // saving at once.
+
+    //FCam::Size (3552,2000)
+    //n900 wide 2560*1440
+
+    int luckyCount = 0;
+    FCam::Frame sharpest;
+    int sharpestScore = 0;
+
     FCam::Shot photo;
-    photo.image = FCam::Image(sensor.maxImageSize(), FCam::RAW, FCam::Image::AutoAllocate);
-    
-    QQueue<QString> pictureNames;
+    FCam::Size wideSize, photoSize;
+
+    int failedTries = 0;
+    int timeSinceLastAccelerometerValue = 0;
+    pAccelerometer::ReadingVal oldAccReading, accReading;
+
+//    QQueue<QString> pictureNames;
 //    LEDBlinker blinker;
 //    LEDBlinker::BlinkAction blink(&blinker);
 
     while (keepGoing) {
 	while (active) {
+#ifdef Q_WS_MAEMO_5
+    wideSize = FCam::Size(2560,1440);
+#else
+    wideSize = FCam::Size(3552,2000);
+#endif
+    if (parameters->aspectRatio.mode==parameters->aspectRatio.WIDE) photoSize = wideSize; else photoSize = sensor.maxImageSize();
+    photo.image = FCam::Image(photoSize, FCam::RAW, FCam::Image::AutoAllocate);
     #ifdef Q_WS_MAEMO_5
         FCam::Event e;
                 while (FCam::getNextEvent(&e)) {
@@ -179,7 +202,7 @@ void CameraThread::run() {
         }
 		
 		// Take a picture once autofocus completes and we have space to store the frame
-		if (takeSnapshot && autoFocus.idle() && writer.savesPending() < 8) {
+        if (takeSnapshot && autoFocus.idle() && writer->savesPending() < 8) {
 		    // use the metering the viewfinder has been doing
 		    photo.exposure  = viewfinder.exposure;
 		    photo.gain      = viewfinder.gain;
@@ -188,21 +211,66 @@ void CameraThread::run() {
             else photo.clearActions();
             feedback.time = photo.exposure;
             photo.addAction(feedback);
+            if (parameters->stabilization.mode == CameraParameters::Stabilization::OFF){
+                sensor.capture(photo);
+                takeSnapshot = false;
+            } else if (parameters->stabilization.mode == CameraParameters::Stabilization::BURST) {
+                // Save the sharpest of 8
+                std::vector<FCam::Shot> burst;
+                burst.resize(8);
+                for (int i = 0; i < 8; i++) {
+                    burst[i] = photo;
+                    burst[i].frameTime = 250000;
+                    burst[i].id = SHARPEST;
+                }
+                sensor.capture(burst);
+                luckyCount = 0;
+                takeSnapshot = false;
+            } else if (parameters->stabilization.mode == CameraParameters::Stabilization::ACCELEROMETER || parameters->stabilization.mode == CameraParameters::Stabilization::BOTH) {
+                // 10hz = once every 10000μs // interestingly it always returns zero difference under XXX
+                timeSinceLastAccelerometerValue += viewfinder.exposure;
+                qDebug()<<timeSinceLastAccelerometerValue;
+                if (timeSinceLastAccelerometerValue > 80000){
+                    oldAccReading = accReading;
+                    timeSinceLastAccelerometerValue = 0;
+                    accReading = accelerometer->reading();
+                    if    (qFabs(accReading.x - oldAccReading.x) < 0.1 + 0.007*(float)failedTries  &&
+                           qFabs(accReading.y - oldAccReading.y) < 0.1 + 0.007*(float)failedTries &&
+                           qFabs(accReading.z - oldAccReading.z) < 0.1 + 0.007*(float)failedTries ){
+                        //qDebug()<< qFabs(accReading.y - oldAccReading.y);
+                        if (parameters->stabilization.mode == CameraParameters::Stabilization::BOTH){
+                            // Save the sharpest of 4
+                            std::vector<FCam::Shot> burst;
+                            burst.resize(8);
+                            for (int i = 0; i < 8; i++) {
+                                burst[i] = photo;
+                                burst[i].frameTime = 500000;
+                                burst[i].id = SHARPEST;
+                            }
+                            sensor.capture(burst);
+                            luckyCount = 0;
+                        } else {
+                            sensor.capture(photo);
+                        }
+                        takeSnapshot = false;
+                        failedTries = 0;
+                    } else {
+                        failedTries++;
+                    }
+                }
+            }
 
-		    sensor.capture(photo);
-		    takeSnapshot = false;
 		}
-
-		// Drain the queue
-		FCam::Frame f;
+        // Drain the queue
+        FCam::Frame f;
 		do {
             f = sensor.getFrame();
             // filewriter doesn't emit a signal when it finishes saving a file so we use a hackish way;
             //qDebug()<< "before dequeure" <<  pictureNames;
-            if (!pictureNames.isEmpty() && pictureNames.length() > writer.savesPending()) {
-                QString newFile = QString(pictureNames.dequeue());
-                if (newFile.endsWith(".jpg")) emit pictureSaved(newFile);
-            }
+//            if (!pictureNames.isEmpty() && pictureNames.length() > writer.savesPending()) {
+//                QString newFile = QString(pictureNames.dequeue());
+//                if (newFile.endsWith(".jpg")) emit pictureSaved(newFile);
+//            }
 
 		    if (f.shot().id == photo.id) {
                 // Our photo came back, asynchronously save it to disk
@@ -216,25 +284,39 @@ void CameraThread::run() {
                     printf("Got a full-res frame\n");
                 }
 
-                QSettings settings;
-                char fname[256];
-                // Save it as a JPEG
-                snprintf(fname, 255, "%s/MyDocs/DCIM/photo_%s.jpg", getenv("HOME"),
-                     f.exposureStartTime().toString().c_str());
-                writer.saveJPEG(f, fname, 90);
-                // filewriter doesn't emit a signal when it finishes saving a file so we use a hackish way;
-                pictureNames.enqueue(QString(fname));
-                //qDebug()<< "after enqueue" <<  pictureNames;
+                writer->save(f);
+            } else if (f.shot().id == SHARPEST) {
+                // evaluate the sharpness of this frame
+                FCam::Image im = f.image();
+                int sharpness = 0;
+                if (im.valid()) {
+                    for (size_t y = 20; y < im.height()-20; y += 20) {
+                        for (size_t x = 20; x < im.width()-20; x += 20) {
+                            sharpness += abs(im(x, y)[1] - im(x-1, y)[1]);
+                            sharpness += abs(im(x, y)[1] - im(x+1, y)[1]);
+                            sharpness += abs(im(x, y)[1] - im(x, y-1)[1]);
+                            sharpness += abs(im(x, y)[1] - im(x, y+1)[1]);
+                        }
+                    }
+                }
+                printf("Frame %d had sharpness %d\n", luckyCount, sharpness);
 
-                // Save it as a DNG
-                snprintf(fname, 255, "%s/MyDocs/DCIM/photo_%s.dng", getenv("HOME"),
-                 f.exposureStartTime().toString().c_str());
-                if (settings.value("saveDng",true)== true) {
-                    writer.saveDNG(f, fname);
-                    pictureNames.enqueue(QString(fname));
+                // Decide whether it's better than the best so far
+                if (luckyCount == 0 || sharpness > sharpestScore) {
+                    printf("Accepting as sharper\n");
+                    sharpest = f;
+                    sharpestScore = sharpness;
+                } else {
+                    printf("Rejecting as blurrier\n");
+                }
+                // Save the best one in the burst
+                if (luckyCount == 7) {
+                    writer->save(sharpest);
+                    sharpest = FCam::Frame();
                 }
 
-		    } else if (f.shot().id == viewfinder.id) {
+                luckyCount++;
+            } else if (f.shot().id == viewfinder.id) {
 
 			// update the autofocus and metering algorithms
             autoFocus.update(f);
@@ -290,12 +372,24 @@ void CameraThread::run() {
 		} while (sensor.framesPending());
 	}
 	sleep(1);
+//#ifdef Q_WS_MAEMO_5
+    // It is necessary to capture one "high resolution" image when exiting, otherwise camera-ui may not work.
+//    sensor.capture(photo);
+//    FCam::Frame fhr = sensor.getFrame();
+//    printf("got high res, exiting now\n");
+//#endif
 	sensor.stop();
 	cameralock->acquire();
 	cameralock->release();
     }
-    // It is necessary to capture one "high resolution" image when exiting, otherwise camera-ui may not work.
-    sensor.capture(photo);
-    FCam::Frame fhr = sensor.getFrame();
-    printf("got high res, exiting now\n");
+//        // It is necessary to capture one "high resolution" image when exiting, otherwise camera-ui may not work.
+//        sensor.start();
+//        sensor.capture(photo);
+//        FCam::Frame fhr = sensor.getFrame();
+//        sensor.stop();
+//        printf("got high res, exiting now\n");
+//    delete accelerometer;
+
+    emit finished();
+    qDebug()<<"out";
 }
